
import edu.illinois.cs.cs125.gradlegrader.annotations.Graded;
import edu.illinois.cs.cs125.gradlegrader.annotations.Tag;
import org.junit.Test;
import java.io.PrintStream;
import java.security.GeneralSecurityException;
import java.io.ByteArrayOutputStream;
import java.util.Arrays;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;


public class GenomTests {



	private char[] gs1 = {'A', 'C', 'G', 'T'};
	private char[] gs2 = {'A', 'C', 'G', 'T'};
	private char[] gs3 = {'A', 'C', 'G', 'Q'};
	private Genom g1 = new Genom(gs1);
	private Genom g2 = new Genom(gs2);
	private Genom g3 = new Genom(gs3);


    @Test(timeout = 1000)
    @Graded(points = 1)
    @Tag(name = "difficulty", value = "simple")
    @Tag(name = "function", value = "testGenomCreationAndEquals")
    public void testGenomCreationAndEquals() {
		
		testCreation();
		testEquals();
    }

	@Test
	public void testCreation() {
		Genom g1 = new Genom(10);
		Genom g2 = new Genom(10);
		assertNotEquals(g1, g2);
	}	


	@Test
	public void testEquals() {
		assertTrue(g1.isEqual(g2));
		assertNotEquals(g1, g3);
	}


	@Test(timeout = 1000)
    @Graded(points = 2)
    @Tag(name = "difficulty", value = "simple")
    @Tag(name = "function", value = "testGenomMutation")
	public void testGenomMutation() {

		testPointMutation();
		testDeletion();
		testInsertion();
	}



	@Test 
	public void testPointMutation() { 
		
		Genom gNew = g1.clone();
		boolean oneWasDifferent = false;
		for (int i = 0; i < 10; i++) {
			gNew.pointMutation();
			if (g1 != gNew) {
				oneWasDifferent = true;
			}

		}
		assertTrue(oneWasDifferent);

	}

	@Test 
	public void testInsertion() {
		Genom gNew = g1.clone();
		gNew.insertion();
		
		assertEquals(g1.getLength()  + 1 , gNew.getLength());
		int i = 0; 
		while (i < g1.getLength() && g1.getAASequence()[i] == gNew.getAASequence()[i]) {
			i++;
		}
		
		while (i < g1.getLength()) {
			assertEquals(g1.getAASequence()[i], gNew.getAASequence()[i + 1]);
			i++;
		}
	}

	@Test
	public void testDeletion() { 
		Genom gNew = g1.clone();
		gNew.deletion();


		assertEquals(g1.getLength() - 1, gNew.getLength());
		
		int i = 0; 
		while (i < gNew.getLength() && g1.getAASequence()[i] == gNew.getAASequence()[i]) {
			i++;
		}
		i += 1;
		while (i < g1.getLength()) {
			assertEquals(g1.getAASequence()[i], gNew.getAASequence()[i - 1]);
			i++;
		}
	}


	@Test(timeout = 1000)
    @Graded(points = 1)
    @Tag(name = "difficulty", value = "simple")
    @Tag(name = "function", value = "testGenePool")
	public void testGenePool() {

		testMutatePool();
		testContainsGeneInPool();
	}

	@Test
	public void testMutatePool() {
		GenePool pool = new GenePool(10, 10);
		String poolAsStringBeforeMutation = pool.toString();
		int sizeBoforeMutation = pool.getGenoms().length;
		pool.mutate();		
		String poolAsStringAfterMutation = pool.toString();
		int sizeAfterMutation = pool.getGenoms().length;
		assertNotEquals(poolAsStringBeforeMutation, poolAsStringAfterMutation);
		assertEquals(sizeBoforeMutation, sizeAfterMutation);
	}

	public void testContainsGeneInPool() {
		GenePool pool = new GenePool(10, 10);
		Genom[] genomes = pool.getGenoms();

		for (Genom g : genomes) {
			assertTrue(pool.contains(g));
		}

		Genom newGenom = genomes[0].clone();
		newGenom.insertion();
		newGenom.pointMutation();
		assertFalse(pool.contains(newGenom));

	}

}
